@@ -1,12 +1,11 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import operator
 import math
+import operator
 
 from gi.repository import Gdk, GLib, GObject, Gtk
 
-from tryton.common import RPCExecute, RPCException
-from tryton.common import eval_domain
+from tryton.common import RPCException, RPCExecute, eval_domain
 
 
 class SelectionMixin(object):
@@ -49,6 +48,8 @@ class SelectionMixin(object):
     def update_selection(self, record, field):
         if not field:
             return
+        if not self.selection:
+            self.init_selection()
 
         domain = field.domain_get(record)
         if 'relation' not in self.attrs:
@@ -72,7 +73,7 @@ class SelectionMixin(object):
             try:
                 result = RPCExecute('model', self.attrs['relation'],
                     'search_read', domain, 0, None, None, fields,
-                    context=context)
+                    context=context, process_exception=False)
             except RPCException:
                 result = False
             if isinstance(result, list):
