@@ -1,29 +1,65 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import configparser
+import gettext
+import locale
+import logging
 import optparse
 import os
-import gettext
-import logging
+import shutil
 import sys
-import locale
+import tempfile
 
 from gi.repository import GdkPixbuf
 
 from tryton import __version__
 
+logger = logging.getLogger(__name__)
 _ = gettext.gettext
 
 
-def get_config_dir():
+def _reverse_series_iterator(starting_version):
+    major, minor = map(int, starting_version.split('.'))
+    while major >= 0 and minor >= 0:
+        yield f"{major}.{minor}"
+        if minor == 0:
+            major -= 1
+            minor = 8
+        else:
+            minor -= 2 if not minor % 2 else 1
+
+
+def copy_previous_configuration(config_element):
+    current_version = __version__.rsplit('.', 1)[0]
+    config_dir = get_config_root()
+    for version in _reverse_series_iterator(current_version):
+        config_path = os.path.join(config_dir, version, config_element)
+        if version == current_version and os.path.exists(config_path):
+            break
+        elif os.path.exists(config_path):
+            if os.path.isfile(config_path):
+                shutil.copy(config_path, get_config_dir())
+            elif os.path.isdir(config_path):
+                shutil.copytree(
+                    config_path,
+                    os.path.join(get_config_dir(), config_element))
+            break
+
+
+def get_config_root():
     if os.name == 'nt':
         appdata = os.environ['APPDATA']
         if not isinstance(appdata, str):
             appdata = str(appdata, sys.getfilesystemencoding())
-        return os.path.join(appdata, '.config', 'tryton',
-                __version__.rsplit('.', 1)[0])
-    return os.path.join(os.environ['HOME'], '.config', 'tryton',
-            __version__.rsplit('.', 1)[0])
+        config_path = os.path.join(appdata, '.config')
+    else:
+        config_path = os.path.expanduser(os.getenv(
+            'XDG_CONFIG_HOME', os.path.join('~', '.config')))
+    return os.path.join(config_path, 'tryton')
+
+
+def get_config_dir():
+    return os.path.join(get_config_root(), __version__.rsplit('.', 1)[0])
 
 
 os.makedirs(get_config_dir(), mode=0o700, exist_ok=True)
@@ -39,6 +75,8 @@ class ConfigManager(object):
         self.defaults = {
             'login.profile': demo_server,
             'login.login': 'demo',
+            'login.service': '',
+            'login.service.port': 8001,
             'login.host': demo_server,
             'login.db': demo_database,
             'login.expanded': False,
@@ -48,6 +86,7 @@ class ConfigManager(object):
             'client.save_tree_width': True,
             'client.save_tree_state': True,
             'client.spellcheck': False,
+            'client.code_scanner_sound': True,
             'client.lang': locale.getdefaultlocale()[0],
             'client.language_direction': 'ltr',
             'client.email': '',
@@ -60,10 +99,11 @@ class ConfigManager(object):
             'graph.color': '#3465a4',
             'image.max_size': 10 ** 6,
             'image.cache_size': 1024,
+            'doc.url': 'https://docs.tryton.org/%(version)s',
             'bug.url': 'https://bugs.tryton.org/',
             'download.url': 'https://downloads.tryton.org/',
             'download.frequency': 60 * 60 * 8,
-            'menu.pane': 200,
+            'menu.pane': 320,
         }
         self.config = {}
         self.options = {}
@@ -83,17 +123,24 @@ class ConfigManager(object):
         parser.add_option("-l", "--log-level", dest="log_level",
                 help=_("specify the log level: "
                 "DEBUG, INFO, WARNING, ERROR, CRITICAL"))
+        parser.add_option("-o", "--log-ouput", dest="log_output", default=None,
+            help=_("specify the file used to output logging information"))
         parser.add_option("-u", "--user", dest="login",
                 help=_("specify the login user"))
         parser.add_option("-s", "--server", dest="host",
                 help=_("specify the server hostname:port"))
+        parser.add_option(
+            '--no-thread', default=True, action='store_false', dest='thread',
+            help=_("disable thread usage"))
         opt, self.arguments = parser.parse_args()
         self.rcfile = opt.config or os.path.join(
             get_config_dir(), 'tryton.conf')
         self.load()
 
-        self.options['dev'] = opt.dev
-        logging.basicConfig()
+        logging_config = {}
+        if opt.log_output:
+            logging_config['filename'] = opt.log_output
+
         loglevels = {
             'DEBUG': logging.DEBUG,
             'INFO': logging.INFO,
@@ -106,11 +153,14 @@ class ConfigManager(object):
                 opt.log_level = 'INFO'
             else:
                 opt.log_level = 'ERROR'
-        logging.getLogger().setLevel(loglevels[opt.log_level.upper()])
+        logging_config['level'] = loglevels[opt.log_level.upper()]
+        logging.basicConfig(**logging_config)
 
+        self.options['dev'] = opt.dev
         for arg in ['login', 'host']:
             if getattr(opt, arg):
                 self.options['login.' + arg] = getattr(opt, arg)
+        self.options['thread'] = opt.thread
 
     def save(self):
         try:
@@ -125,15 +175,25 @@ class ConfigManager(object):
             with open(self.rcfile, 'w') as fp:
                 parser.write(fp)
         except IOError:
-            logging.getLogger(__name__).warn(
-                _('Unable to write config file %s.')
-                % (self.rcfile,))
+            logger.warn("Unable to write config file %s", self.rcfile)
             return False
         return True
 
     def load(self):
         parser = configparser.ConfigParser()
-        parser.read([self.rcfile])
+        try:
+            parser.read([self.rcfile])
+        except configparser.Error:
+            config_dir = os.path.dirname(self.rcfile)
+            with tempfile.NamedTemporaryFile(
+                    delete=False, prefix='tryton_', suffix='.conf',
+                    dir=config_dir) as temp_file:
+                temp_name = temp_file.name
+            shutil.copy(self.rcfile, temp_name)
+            logger.error(
+                f"Failed to parse {self.rcfile}. "
+                f"A backup can be found at {temp_name}", exc_info=True)
+            return
         for section in parser.sections():
             for (name, value) in parser.items(section):
                 if value.lower() == 'true':
@@ -164,10 +224,25 @@ if not isinstance(CURRENT_DIR, str):
 
 PIXMAPS_DIR = os.path.join(CURRENT_DIR, 'data', 'pixmaps', 'tryton')
 if not os.path.isdir(PIXMAPS_DIR):
-    # do not import when frozen
-    import pkg_resources
-    PIXMAPS_DIR = pkg_resources.resource_filename(
-        'tryton', 'data/pixmaps/tryton')
+    try:
+        import importlib.resources
+        ref = importlib.resources.files('tryton') / 'data/pixmaps/tryton'
+        with importlib.resources.as_file(ref) as path:
+            PIXMAPS_DIR = path
+    except ImportError:
+        import pkg_resources
+        PIXMAPS_DIR = pkg_resources.resource_filename(
+            'tryton', 'data/pixmaps/tryton')
+SOUNDS_DIR = os.path.join(CURRENT_DIR, 'data', 'sounds')
+if not os.path.isdir(SOUNDS_DIR):
+    try:
+        import importlib.resources
+        ref = importlib.resources.files('tryton') / 'data/sounds'
+        with importlib.resources.as_file(ref) as path:
+            SOUNDS_DIR = path
+    except ImportError:
+        import pkg_resources
+        SOUNDS_DIR = pkg_resources.resource_filename('tryton', 'data/sounds')
 
 TRYTON_ICON = GdkPixbuf.Pixbuf.new_from_file(
     os.path.join(PIXMAPS_DIR, 'tryton-icon.png'))
