@@ -130,20 +130,16 @@ class Field(object):
                 if value is False:
                     # XXX to remove once server domains are fixed
                     value = None
-                setdefault = True
                 if record.group.domain:
                     original_domain = merge(record.group.domain)
                 else:
                     original_domain = merge(domain)
                 domain_readonly = original_domain[0] == 'AND'
                 if '.' in leftpart:
-                    recordpart, localpart = leftpart.split('.', 1)
-                    constraintfields = set()
-                    if domain_readonly:
-                        for leaf in localize_domain(original_domain[1:]):
-                            constraintfields.add(leaf[0])
-                    if localpart != 'id' or recordpart not in constraintfields:
-                        setdefault = False
+                    localpart = leftpart.split('.', 1)[1]
+                    setdefault = localpart == 'id'
+                else:
+                    setdefault = True
                 if setdefault and not pre_validate:
                     self.set_client(record, value)
                     state_attrs['domain_readonly'] = domain_readonly
@@ -451,10 +447,20 @@ class FloatField(Field):
 
     def apply_factor(self, record, value, factor):
         if value is not None:
+            # The default precision is the one used by value (before
+            # applying the factor), as python uses the shortest representation
+            # of said value.
+            # Once the factor is applied the number might become even
+            # more inexact thus we should rely on the initial
+            # precision + the effect factor will have
+            default_precision = len((str(value).split('.', 2) + [''])[1])
+            default_precision += math.ceil(math.log10(factor))
             value /= factor
             digits = self.digits(record)
             if digits:
                 value = round(value, digits[1])
+            else:
+                value = round(value, default_precision)
             value = self.convert(value)
         return value
 
@@ -833,6 +839,7 @@ class O2MField(Field):
                         force_remove=False)
 
         if value and (value.get('add') or value.get('update', [])):
+            vals_to_set = {}
             # First set already added fields to prevent triggering a
             # second on_change call
             for vals in value.get('update', []):
@@ -860,7 +867,10 @@ class O2MField(Field):
                     continue
                 record2 = group.get(vals['id'])
                 if record2 is not None:
-                    record2.set_on_change(vals)
+                    to_set = {
+                        k: v for k, v in vals.items()
+                        if k not in vals_to_set}
+                    record2.set_on_change(to_set)
 
     def validation_domains(self, record, pre_validate=None):
         screen_domain, attr_domain = self.domains_get(record, pre_validate)
