@@ -1,14 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import gettext
-import urllib.request
 
 from gi.repository import Gdk, Gtk
 
-from tryton.common import resize_pixbuf, data2pixbuf
+from tryton.common import data2pixbuf, resize_pixbuf, url_open
 from tryton.config import CONFIG
-from .widget import Widget
+
 from .binary import BinaryMixin
+from .widget import Widget
 
 _ = gettext.gettext
 
@@ -21,7 +21,11 @@ class Image(BinaryMixin, Widget):
         self.height = int(attrs.get('height', 100))
         self.width = int(attrs.get('width', 300))
 
-        self.widget = Gtk.VBox(spacing=3)
+        self.widget = Gtk.Frame()
+        self.widget.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        self.widget.get_accessible().set_name(attrs.get('string', ''))
+        vbox = Gtk.VBox(spacing=3, margin=5)
+        self.widget.add(vbox)
         self.event = Gtk.EventBox()
         self.event.drag_dest_set(Gtk.DestDefaults.ALL, [
             Gtk.TargetEntry.new('text/plain', 0, 0),
@@ -33,12 +37,12 @@ class Image(BinaryMixin, Widget):
 
         self.image = Gtk.Image()
         self.event.add(self.image)
-        self.widget.pack_start(self.event, expand=True, fill=True, padding=0)
+        vbox.pack_start(self.event, expand=True, fill=True, padding=0)
 
         toolbar = self.toolbar()  # Set button attributes even if not display
         if not attrs.get('readonly'):
             toolbar.set_halign(Gtk.Align.CENTER)
-            self.widget.pack_start(
+            vbox.pack_start(
                 toolbar, expand=False, fill=False, padding=0)
 
         self._readonly = False
@@ -77,14 +81,12 @@ class Image(BinaryMixin, Widget):
         if info == 0:
             uri = selection.get_text().split('\n')[0]
             if uri:
-                self.field.set_client(
-                    self.record, urllib.request.urlopen(uri).read())
+                self.field.set_client(self.record, url_open(uri).read())
             self.update_img()
         elif info == 1:
             uri = selection.get_data().split('\r\n')[0]
             if uri:
-                self.field.set_client(
-                    self.record, urllib.request.urlopen(uri).read())
+                self.field.set_client(self.record, url_open(uri).read())
             self.update_img()
         elif info == 2:
             data = selection.get_pixbuf()
