@@ -5,13 +5,14 @@ import gettext
 from gi.repository import Gdk, Gtk
 
 import tryton.common as common
-from .widget import Widget
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.domain_parser import quote
 from tryton.common.underline import set_underline
 from tryton.gui.window.view_form.screen import Screen
-from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
+from tryton.gui.window.win_search import WinSearch
+
+from .widget import Widget
 
 _ = gettext.gettext
 
@@ -52,8 +53,8 @@ class Many2Many(Widget):
 
         if int(self.attrs.get('completion', 1)):
             self.wid_completion = get_completion(
-                create=self.attrs.get('create', True)
-                and common.MODELACCESS[self.attrs['relation']]['create'])
+                search=self.read_access,
+                create=self.create_access)
             self.wid_completion.connect('match-selected',
                 self._completion_match_selected)
             self.wid_completion.connect('action-activated',
@@ -71,6 +72,9 @@ class Many2Many(Widget):
         self.but_add.set_relief(Gtk.ReliefStyle.NONE)
         hbox.pack_start(self.but_add, expand=False, fill=False, padding=0)
 
+        self.label = Gtk.Label(label='(_/0)')
+        hbox.pack_start(self.label, expand=False, fill=False, padding=0)
+
         self.but_remove = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_remove, _('Remove selected record'))
         self.but_remove.connect('clicked', self._sig_remove)
@@ -99,7 +103,7 @@ class Many2Many(Widget):
             limit=None,
             context=self.view.screen.context,
             breadcrumb=breadcrumb)
-        self.screen.signal_connect(self, 'record-message', self._sig_label)
+        self.screen.windows.append(self)
 
         vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
 
@@ -141,6 +145,21 @@ class Many2Many(Widget):
         self.wid_text.disconnect_by_func(self._focus_out)
         self.screen.destroy()
 
+    def get_access(self, type_):
+        model = self.attrs['relation']
+        if model:
+            return common.MODELACCESS[model][type_]
+        else:
+            return True
+
+    @property
+    def read_access(self):
+        return self.get_access('read')
+
+    @property
+    def create_access(self):
+        return int(self.attrs.get('create', 1)) and self.get_access('create')
+
     def _sig_add(self, *args):
         if not self.focus_out:
             return
@@ -159,23 +178,16 @@ class Many2Many(Widget):
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
-                self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
         win = WinSearch(self.attrs['relation'], callback, sel_multi=True,
             context=context, domain=domain, order=order,
             view_ids=self.attrs.get('view_ids', '').split(','),
             views_preload=self.attrs.get('views', {}),
-            new=self.attrs.get('create', True),
+            new=self.create_access,
             title=self.attrs.get('string'))
         win.screen.search_filter(quote(value))
-        if len(win.screen.group) == 1:
-            callback([
-                    (r.id, r.value.get('rec_name', ''))
-                    for r in win.screen.group])
-            win.destroy()
-        else:
-            win.show()
+        win.show()
 
     def _sig_remove(self, *args):
         self.screen.remove(remove=True)
@@ -207,6 +219,7 @@ class Many2Many(Widget):
         # save of the record will trigger the save of the parent
         screen = self._get_screen_form()
         screen.load([self.screen.current_record.id])
+        screen.current_record = screen.group.get(self.screen.current_record.id)
 
         def callback(result):
             if result:
@@ -220,8 +233,10 @@ class Many2Many(Widget):
                 self.screen.display()
         WinForm(screen, callback)
 
-    def _sig_new(self):
+    def _sig_new(self, defaults=None):
         screen = self._get_screen_form()
+        defaults = defaults.copy() if defaults is not None else {}
+        defaults['rec_name'] = self.wid_text.get_text()
 
         def callback(result):
             self.focus_out = True
@@ -232,8 +247,8 @@ class Many2Many(Widget):
             self.wid_text.grab_focus()
 
         self.focus_out = False
-        WinForm(screen, callback, new=True, save_current=True,
-            rec_name=self.wid_text.get_text())
+        WinForm(
+            screen, callback, new=True, save_current=True, defaults=defaults)
 
     def _readonly_set(self, value):
         self._readonly = value
@@ -266,8 +281,14 @@ class Many2Many(Widget):
                 not self._readonly
                 and self._position))
 
-    def _sig_label(self, screen, signal_data):
-        self._position = signal_data[0]
+    def record_message(self, position, size, *args):
+        self._position = position
+        name = str(position) if position else '_'
+        selected = len(self.screen.selected_records)
+        if selected > 1:
+            name += '#%i' % selected
+        name = '(%s/%s)' % (name, common.humanize(size))
+        self.label.set_text(name)
         self._set_button_sensitive()
 
     def display(self):
@@ -289,14 +310,17 @@ class Many2Many(Widget):
         return True
 
     def _completion_match_selected(self, completion, model, iter_):
-        record_id, = model.get(iter_, 1)
-        self.screen.load([record_id], modified=True)
-        self.wid_text.set_text('')
-        self.wid_text.grab_focus()
+        record_id, defaults = model.get(iter_, 1, 2)
+        if record_id is not None:
+            self.screen.load([record_id], modified=True)
+            self.wid_text.set_text('')
+            self.wid_text.grab_focus()
 
-        completion_model = self.wid_completion.get_model()
-        completion_model.clear()
-        completion_model.search_text = self.wid_text.get_text()
+            completion_model = self.wid_completion.get_model()
+            completion_model.clear()
+            completion_model.search_text = self.wid_text.get_text()
+        else:
+            self._sig_new(defaults)
         return True
 
     def _update_completion(self, widget):
