@@ -4,20 +4,22 @@ import gettext
 
 from gi.repository import Gdk, GLib, Gtk
 
-from .widget import Widget
 import tryton.common as common
-from tryton.gui.window.view_form.screen import Screen
-from tryton.gui.window.win_search import WinSearch
-from tryton.gui.window.win_form import WinForm
-from tryton.common.popup_menu import populate
 from tryton.common.completion import get_completion, update_completion
-from tryton.common.entry_position import reset_position
 from tryton.common.domain_parser import quote
+from tryton.common.entry_position import reset_position
+from tryton.common.popup_menu import populate
+from tryton.gui.window.view_form.screen import Screen
+from tryton.gui.window.win_form import WinForm
+from tryton.gui.window.win_search import WinSearch
+
+from .widget import Widget
 
 _ = gettext.gettext
 
 
 class Many2One(Widget):
+    default_width_chars = 12
 
     def __init__(self, view, attrs):
         super(Many2One, self).__init__(view, attrs)
@@ -74,7 +76,7 @@ class Many2One(Widget):
 
     @property
     def create_access(self):
-        return self.attrs.get('create', True) and self.get_access('create')
+        return int(self.attrs.get('create', 1)) and self.get_access('create')
 
     @property
     def modified(self):
@@ -135,13 +137,7 @@ class Many2One(Widget):
                     title=self.attrs.get('string'),
                     exclude_field=self.attrs.get('relation_field'))
                 win.screen.search_filter(quote(text))
-                if len(win.screen.group) == 1:
-                    callback([
-                            (r.id, r.value.get('rec_name', ''))
-                            for r in win.screen.group])
-                    win.destroy()
-                else:
-                    win.show()
+                win.show()
                 return
         self.focus_out = True
         self.changed = True
@@ -165,12 +161,13 @@ class Many2One(Widget):
             exclude_field=self.attrs.get('relation_field'),
             breadcrumb=breadcrumb)
 
-    def sig_new(self, *args):
-        model = self.get_model()
-        if not model or not common.MODELACCESS[model]['create']:
+    def sig_new(self, defaults=None):
+        if not self.create_access:
             return
         self.focus_out = False
         screen = self.get_screen(search=True)
+        defaults = defaults.copy() if defaults is not None else {}
+        defaults['rec_name'] = self.wid_text.get_text()
 
         def callback(result):
             if result:
@@ -178,8 +175,8 @@ class Many2One(Widget):
                     self.value_from_id(screen.current_record.id,
                         screen.current_record.rec_name()))
             self.focus_out = True
-        WinForm(screen, callback, new=True, save_current=True,
-            rec_name=self.wid_text.get_text())
+        WinForm(
+            screen, callback, new=True, save_current=True, defaults=defaults)
 
     def sig_edit(self, entry=None, icon_pos=None, *args):
         if entry:
@@ -203,8 +200,10 @@ class Many2One(Widget):
             return
 
         if self.has_target(value):
+            m2o_id = self.id_from_value(self.field.get(self.record))
             screen = self.get_screen()
-            screen.load([self.id_from_value(self.field.get(self.record))])
+            screen.load([m2o_id])
+            screen.current_record = screen.group.get(m2o_id)
 
             def callback(result):
                 if result:
@@ -248,7 +247,7 @@ class Many2One(Widget):
         if (event.keyval == Gdk.KEY_F3
                 and editable
                 and self.create_access):
-            self.sig_new(widget, event)
+            self.sig_new()
             return True
         elif event.keyval == Gdk.KEY_F2 and self.read_access:
             self.sig_edit(widget)
@@ -350,15 +349,18 @@ class Many2One(Widget):
         self.wid_text.set_completion(self.wid_completion)
 
     def _completion_match_selected(self, completion, model, iter_):
-        rec_name, record_id = model.get(iter_, 0, 1)
-        # GTK on win32 doesn't like synchronous call to set_client
-        # because it triggers a display which reset the completion
-        GLib.idle_add(self.field.set_client, self.record,
-            self.value_from_id(record_id, rec_name), True)
+        rec_name, record_id, defaults = model.get(iter_, 0, 1, 2)
+        if record_id is not None:
+            # GTK on win32 doesn't like synchronous call to set_client
+            # because it triggers a display which reset the completion
+            GLib.idle_add(self.field.set_client, self.record,
+                self.value_from_id(record_id, rec_name), True)
 
-        completion_model = self.wid_completion.get_model()
-        completion_model.clear()
-        completion_model.search_text = rec_name
+            completion_model = self.wid_completion.get_model()
+            completion_model.clear()
+            completion_model.search_text = rec_name
+        else:
+            self.sig_new(defaults)
         return True
 
     def _update_completion(self, widget):
