@@ -5,14 +5,15 @@ import itertools
 
 from gi.repository import Gdk, Gtk
 
-from .widget import Widget
-from tryton.gui.window.view_form.screen import Screen
-from tryton.gui.window.win_search import WinSearch
-from tryton.gui.window.win_form import WinForm
 import tryton.common as common
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.domain_parser import quote
 from tryton.common.underline import set_underline
+from tryton.gui.window.view_form.screen import Screen
+from tryton.gui.window.win_form import WinForm
+from tryton.gui.window.win_search import WinSearch
+
+from .widget import Widget
 
 _ = gettext.gettext
 
@@ -45,13 +46,13 @@ class One2Many(Widget):
 
         tooltips = common.Tooltips()
 
-        but_switch = Gtk.Button(can_focus=False)
-        tooltips.set_tip(but_switch, _('Switch'))
-        but_switch.connect('clicked', self.switch_view)
-        but_switch.add(common.IconFactory.get_image(
+        self.but_switch = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_switch, _('Switch'))
+        self.but_switch.connect('clicked', self.switch_view)
+        self.but_switch.add(common.IconFactory.get_image(
                 'tryton-switch', Gtk.IconSize.SMALL_TOOLBAR))
-        but_switch.set_relief(Gtk.ReliefStyle.NONE)
-        hbox.pack_start(but_switch, expand=False, fill=False, padding=0)
+        self.but_switch.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_switch, expand=False, fill=False, padding=0)
 
         self.but_pre = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_pre, _('Previous'))
@@ -61,7 +62,7 @@ class One2Many(Widget):
         self.but_pre.set_relief(Gtk.ReliefStyle.NONE)
         hbox.pack_start(self.but_pre, expand=False, fill=False, padding=0)
 
-        self.label = Gtk.Label(label='(0,0)')
+        self.label = Gtk.Label(label='(_/0)')
         hbox.pack_start(self.label, expand=False, fill=False, padding=0)
 
         self.but_next = Gtk.Button(can_focus=False)
@@ -85,10 +86,9 @@ class One2Many(Widget):
             hbox.pack_start(self.wid_text, expand=True, fill=True, padding=0)
 
             if int(self.attrs.get('completion', 1)):
-                access = common.MODELACCESS[attrs['relation']]
                 self.wid_completion = get_completion(
-                    search=access['read'] and access['write'],
-                    create=attrs.get('create', True) and access['create'])
+                    search=self.read_access,
+                    create=self.create_access)
                 self.wid_completion.connect('match-selected',
                     self._completion_match_selected)
                 self.wid_completion.connect('action-activated',
@@ -119,7 +119,7 @@ class One2Many(Widget):
 
         self.but_new = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_new, _('Create a new record'))
-        self.but_new.connect('clicked', self._sig_new)
+        self.but_new.connect('clicked', lambda *a: self._sig_new())
         self.but_new.add(common.IconFactory.get_image(
                 'tryton-create', Gtk.IconSize.SMALL_TOOLBAR))
         self.but_new.set_relief(Gtk.ReliefStyle.NONE)
@@ -171,7 +171,7 @@ class One2Many(Widget):
             context=self.view.screen.context,
             breadcrumb=breadcrumb)
         self.screen.pre_validate = bool(int(attrs.get('pre_validate', 0)))
-        self.screen.signal_connect(self, 'record-message', self._sig_label)
+        self.screen.windows.append(self)
 
         vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
 
@@ -182,12 +182,33 @@ class One2Many(Widget):
         if self.attrs.get('add_remove'):
             self.wid_text.connect('key_press_event', self.on_keypress)
 
-        but_switch.props.sensitive = self.screen.number_of_views > 1
+    def get_access(self, type_):
+        model = self.attrs['relation']
+        if model:
+            return common.MODELACCESS[model][type_]
+        else:
+            return True
+
+    @property
+    def read_access(self):
+        return self.get_access('read')
+
+    @property
+    def create_access(self):
+        return int(self.attrs.get('create', 1)) and self.get_access('create')
+
+    @property
+    def write_access(self):
+        return self.get_access('write')
+
+    @property
+    def delete_access(self):
+        return int(self.attrs.get('delete', 1)) and self.get_access('delete')
 
     def on_keypress(self, widget, event):
         if ((event.keyval == Gdk.KEY_F3)
                 and self.but_new.get_property('sensitive')):
-            self._sig_new(widget)
+            self._sig_new()
             return True
         if event.keyval == Gdk.KEY_F2:
             if widget == self.screen.widget:
@@ -260,7 +281,6 @@ class One2Many(Widget):
             self.title, self._readonly, self._required)
 
     def _set_button_sensitive(self):
-        access = common.MODELACCESS[self.screen.model_name]
         if self.record and self.field:
             field_size = self.record.expr_eval(self.attrs.get('size'))
             o2m_size = len(self.field.get_eval(self.record))
@@ -275,25 +295,27 @@ class One2Many(Widget):
             first = self._position <= 1
             last = self._position >= self._length
         deletable = self.screen.deletable
+        view_type = self.screen.current_view.view_type
+        has_views = self.screen.number_of_views > 1
 
+        self.but_switch.set_sensitive(
+            (self._position or view_type == 'form') and has_views)
         self.but_new.set_sensitive(bool(
                 not self._readonly
-                and self.attrs.get('create', True)
-                and not size_limit
-                and access['create']))
+                and self.create_access
+                and not size_limit))
         self.but_del.set_sensitive(bool(
                 not self._readonly
-                and self.attrs.get('delete', True)
+                and self.delete_access
                 and deletable
-                and self._position
-                and access['delete']))
+                and self._position))
         self.but_undel.set_sensitive(bool(
                 not self._readonly
                 and not size_limit
                 and self._position))
         self.but_open.set_sensitive(bool(
                 self._position
-                and access['read']))
+                and self.read_access))
         self.but_next.set_sensitive(bool(
                 self._position
                 and not last))
@@ -304,13 +326,13 @@ class One2Many(Widget):
             self.but_add.set_sensitive(bool(
                     not self._readonly
                     and not size_limit
-                    and access['write']
-                    and access['read']))
+                    and self.write_access
+                    and self.read_access))
             self.but_remove.set_sensitive(bool(
                     not self._readonly
                     and self._position
-                    and access['write']
-                    and access['read']))
+                    and self.write_access
+                    and self.read_access))
             self.wid_text.set_sensitive(self.but_add.get_sensitive())
             self.wid_text.set_editable(self.but_add.get_sensitive())
 
@@ -333,18 +355,21 @@ class One2Many(Widget):
                 if sequence:
                     return sequence
 
-    def _sig_new(self, *args):
-        if not common.MODELACCESS[self.screen.model_name]['create']:
+    def _sig_new(self, defaults=None):
+        if not self.create_access:
             return
         if not self._validate():
             return
+        if self.attrs.get('add_remove'):
+            defaults = defaults.copy() if defaults is not None else {}
+            defaults['rec_name'] = self.wid_text.get_text()
 
         if self.attrs.get('product'):
-            self._new_product()
+            self._new_product(defaults)
         else:
-            self._new_single()
+            self._new_single(defaults)
 
-    def _new_single(self):
+    def _new_single(self, defaults=None):
         sequence = self._sequence()
 
         def update_sequence():
@@ -352,22 +377,23 @@ class One2Many(Widget):
                 self.screen.group.set_sequence(
                     field=sequence, position=self.screen.new_position)
 
-        if self.screen.current_view.editable:
+        if self.screen.current_view.creatable:
             self.screen.new()
             self.screen.current_view.widget.set_sensitive(True)
             update_sequence()
         else:
             field_size = self.record.expr_eval(self.attrs.get('size')) or -1
             field_size -= len(self.field.get_eval(self.record)) + 1
-            WinForm(self.screen, lambda a: update_sequence(), new=True,
-                many=field_size)
+            WinForm(
+                self.screen, lambda a: update_sequence(), new=True,
+                defaults=defaults, many=field_size)
 
-    def _new_product(self):
+    def _new_product(self, defaults=None):
         fields = self.attrs['product'].split(',')
         product = {}
 
         first = self.screen.new(default=False)
-        default = first.default_get()
+        default = first.default_get(defaults=defaults)
         first.set_default(default)
 
         def search_set(*args):
@@ -435,14 +461,13 @@ class One2Many(Widget):
         self.screen.display_prev()
 
     def _sig_remove(self, widget, remove=False):
-        access = common.MODELACCESS[self.screen.model_name]
         writable = not self.screen.readonly
         deletable = self.screen.deletable
         if remove:
-            if not access['write'] or not writable or not access['read']:
+            if not self.write_access or not writable or not self.read_access:
                 return
         else:
-            if not access['delete'] or not deletable:
+            if not self.delete_access or not deletable:
                 return
         self.screen.remove(remove=remove)
 
@@ -452,8 +477,7 @@ class One2Many(Widget):
     def _sig_add(self, *args):
         if not self.focus_out:
             return
-        access = common.MODELACCESS[self.screen.model_name]
-        if not access['write'] or not access['read']:
+        if not self.write_access or not self.read_access:
             return
         self.view.set_value()
         domain = self.field.domain_get(self.record)
@@ -472,7 +496,6 @@ class One2Many(Widget):
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
-                self.screen.display(res_id=ids[0])
                 if sequence:
                     self.screen.group.set_sequence(
                         field=sequence, position=self.screen.new_position)
@@ -490,15 +513,15 @@ class One2Many(Widget):
         win.screen.search_filter(quote(text))
         win.show()
 
-    def _sig_label(self, screen, signal_data):
-        self._position = signal_data[0]
-        self._length = signal_data[1]
-        if self._position:
-            name = str(self._position)
-        else:
-            name = '_'
-        line = '(%s/%s)' % (name, self._length)
-        self.label.set_text(line)
+    def record_message(self, position, size, *args):
+        self._position = position
+        self._length = size
+        name = str(position) if position else '_'
+        selected = len(self.screen.selected_records)
+        if selected > 1:
+            name += '#%i' % selected
+        name = '(%s/%s)' % (name, common.humanize(size))
+        self.label.set_text(name)
         self._set_button_sensitive()
 
     def display(self):
@@ -516,15 +539,15 @@ class One2Many(Widget):
 
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
-            if (self.screen.current_view.view_type == 'tree') \
-                    and self.screen.current_view.editable:
-                self.screen.current_record = None
+            if (self.screen.current_view.view_type == 'form'
+                    and self.screen.group):
+                self.screen.current_record = self.screen.group[0]
         domain = []
         size_limit = None
         if self.record:
             domain = self.field.domain_get(self.record)
             size_limit = self.record.expr_eval(self.attrs.get('size'))
-        if self._readonly:
+        if self._readonly or not self.create_access:
             if size_limit is None:
                 size_limit = len(self.screen.group)
             else:
@@ -538,19 +561,21 @@ class One2Many(Widget):
     def set_value(self):
         self.screen.current_view.set_value()
         if self.screen.modified():  # TODO check if required
-            self.record.modified_fields.setdefault(self.field.name)
-            self.record.signal('record-modified')
+            self.view.screen.record_modified(display=False)
         return True
 
     def _completion_match_selected(self, completion, model, iter_):
-        record_id, = model.get(iter_, 1)
-        self.screen.load([record_id], modified=True)
-        self.wid_text.set_text('')
-        self.wid_text.grab_focus()
+        record_id, defaults = model.get(iter_, 1, 2)
+        if record_id is not None:
+            self.screen.load([record_id], modified=True)
+            self.wid_text.set_text('')
+            self.wid_text.grab_focus()
 
-        completion_model = self.wid_completion.get_model()
-        completion_model.clear()
-        completion_model.search_text = self.wid_text.get_text()
+            completion_model = self.wid_completion.get_model()
+            completion_model.clear()
+            completion_model.search_text = self.wid_text.get_text()
+        else:
+            self._sig_new(defaults)
         return True
 
     def _update_completion(self, widget):
