@@ -1,12 +1,13 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import base64
 import csv
 import datetime
-import os
-import tempfile
 import gettext
 import json
 import locale
+import os
+import tempfile
 import urllib.parse
 from itertools import zip_longest
 from numbers import Number
@@ -14,11 +15,11 @@ from numbers import Number
 from gi.repository import Gdk, GObject, Gtk
 
 import tryton.common as common
+from tryton.common import RPCException, RPCExecute
 from tryton.config import CONFIG
-from tryton.common import RPCExecute, RPCException
 from tryton.gui.window.win_csv import WinCSV
 from tryton.jsonrpc import JSONEncoder
-from tryton.rpc import clear_cache, CONNECTION
+from tryton.rpc import CONNECTION, clear_cache
 
 _ = gettext.gettext
 
@@ -98,15 +99,16 @@ class WinExport(WinCSV):
 
         self.pref_export = Gtk.TreeView()
         self.pref_export.append_column(
-            Gtk.TreeViewColumn(_('Name'), Gtk.CellRendererText(), text=2))
+            Gtk.TreeViewColumn(_('Name'), Gtk.CellRendererText(), text=1))
         scrolledwindow_exports.add(self.pref_export)
         self.pref_export.connect("button-press-event", self.export_click)
         self.pref_export.connect("key-press-event", self.export_keypress)
 
         self.predef_model = Gtk.ListStore(
-                GObject.TYPE_INT,
-                GObject.TYPE_PYOBJECT,
-                GObject.TYPE_STRING)
+            GObject.TYPE_INT,
+            GObject.TYPE_STRING,
+            GObject.TYPE_PYOBJECT,
+            GObject.TYPE_PYOBJECT)
         self.fill_predefwin()
 
     def add_chooser(self, box):
@@ -162,7 +164,7 @@ class WinExport(WinCSV):
                 items.insert(0, ('%s.translated' % name, field,
                         _('%s (model name)') % string_))
                 items.insert(0, ('%s/rec_name' % name, field,
-                        _("%s (record name)") % string_))
+                        _("%s/Record Name") % string_))
 
             for name, field, string_ in items:
                 path = prefix_field + name
@@ -213,17 +215,17 @@ class WinExport(WinCSV):
     def fill_predefwin(self):
         try:
             exports = RPCExecute(
-                'model', 'ir.export', 'search_read',
-                [('resource', '=', self.model)], 0, None, None,
-                ['name', 'export_fields.name'],
+                'model', 'ir.export', 'get', self.model,
+                ['name', 'header', 'records', 'export_fields.name'],
                 context=self.context)
         except RPCException:
             return
         for export in exports:
             self.predef_model.append((
                     export['id'],
+                    export['name'],
                     [f['name'] for f in export['export_fields.']],
-                    export['name']))
+                    export))
         self.pref_export.set_model(self.predef_model)
 
     def addreplace_predef(self, widget):
@@ -247,30 +249,37 @@ class WinExport(WinCSV):
                 return
         else:
             pref_id = model.get_value(iter_, 0)
-            name = model.get_value(iter_, 2)
+            name = model.get_value(iter_, 1)
             override = common.sur(_("Override '%s' definition?") % name)
             if not override:
                 return
         try:
+            values = {
+                'header': self.add_field_names.get_active(),
+                'records': (
+                    'selected' if self.selected_records.get_active()
+                    else 'listed'),
+                }
             if not pref_id:
-                pref_id, = RPCExecute('model', 'ir.export', 'create', [{
+                values.update({
                         'name': name,
                         'resource': self.model,
-                        'export_fields': [('create', [{
-                                            'name': x,
-                                            } for x in fields])],
-                        }], context=self.context)
+                        'export_fields': [{'name': x, } for x in fields],
+                        })
+                pref_id = RPCExecute(
+                    'model', 'ir.export', 'set', values, context=self.context)
             else:
-                RPCExecute('model', 'ir.export', 'update', [pref_id], fields,
+                RPCExecute(
+                    'model', 'ir.export', 'update', pref_id, values, fields,
                     context=self.context)
         except RPCException:
             return
         clear_cache('model.%s.view_toolbar_get' % self.model)
         if iter_ is None:
-            self.predef_model.append((pref_id, fields, name))
+            self.predef_model.append((pref_id, name, fields, values))
         else:
-            model.set_value(iter_, 0, pref_id)
-            model.set_value(iter_, 1, fields)
+            model.set_value(iter_, 2, fields)
+            model.set_value(iter_, 3, values)
 
     def remove_predef(self, widget):
         sel = self.pref_export.get_selection().get_selected()
@@ -281,8 +290,8 @@ class WinExport(WinCSV):
             return None
         export_id = model.get_value(i, 0)
         try:
-            RPCExecute('model', 'ir.export', 'delete', [export_id],
-                context=self.context)
+            RPCExecute(
+                'model', 'ir.export', 'unset', export_id, context=self.context)
         except RPCException:
             return
         clear_cache('model.%s.view_toolbar_get' % self.model)
@@ -294,7 +303,8 @@ class WinExport(WinCSV):
 
     def sel_predef(self, path):
         self.model2.clear()
-        for name in self.predef_model[path[0]][1]:
+        _, _, fields, values = self.predef_model[path[0]]
+        for name in fields:
             if name not in self.fields:
                 iter = self.model1.get_iter_first()
                 prefix = ''
@@ -314,6 +324,10 @@ class WinExport(WinCSV):
                 continue
             self.sel_field(name)
 
+        self.add_field_names.set_active(values.get('header'))
+        self.selected_records.set_active(
+            int(values.get('records') == 'selected'))
+
     def sel_field(self, name):
         string_, relation = self.fields[name]
         if relation:
@@ -323,12 +337,11 @@ class WinExport(WinCSV):
     def response(self, dialog, response):
         if response == Gtk.ResponseType.OK:
             fields = []
-            fields2 = []
             iter = self.model2.get_iter_first()
             while iter:
                 fields.append(self.model2.get_value(iter, 1))
-                fields2.append(self.model2.get_value(iter, 0))
                 iter = self.model2.iter_next(iter)
+            header = self.add_field_names.get_active()
 
             if self.selected_records.get_active():
                 ids = [r.id for r in self.screen.selected_records]
@@ -336,7 +349,7 @@ class WinExport(WinCSV):
                 try:
                     data = RPCExecute(
                         'model', self.model, 'export_data',
-                        ids, fields,
+                        ids, fields, header,
                         context=self.context)
                 except RPCException:
                     data = []
@@ -346,7 +359,7 @@ class WinExport(WinCSV):
                 try:
                     data = RPCExecute(
                         'model', self.model, 'export_data',
-                        ids, fields,
+                        ids, fields, header,
                         context=self.context)
                 except RPCException:
                     data = []
@@ -362,7 +375,7 @@ class WinExport(WinCSV):
                     data = RPCExecute(
                         'model', self.model, 'export_data_domain',
                         domain, fields, offset, limit, self.screen.order,
-                        context=self.context)
+                        header, context=self.context)
                 except RPCException:
                     data = []
 
@@ -370,17 +383,20 @@ class WinExport(WinCSV):
                 fname = common.file_selection(_('Save As...'),
                         action=Gtk.FileChooserAction.SAVE)
                 if fname:
-                    self.export_csv(fname, fields2, data, paths)
+                    self.export_csv(fname, data, paths, header=header)
             else:
                 fileno, fname = tempfile.mkstemp(
                     '.csv', common.slugify(self.name) + '_')
-                self.export_csv(fname, fields2, data, paths, popup=False)
-                os.close(fileno)
-                common.file_open(fname, 'csv')
+                if self.export_csv(
+                        fname, data, paths, popup=False, header=header):
+                    os.close(fileno)
+                    common.file_open(fname, 'csv')
+                else:
+                    os.close(fileno)
         self.destroy()
 
-    def export_csv(self, fname, fields, data, paths, popup=True):
-        encoding = self.csv_enc.get_active_text() or 'UTF-8'
+    def export_csv(self, fname, data, paths, popup=True, header=False):
+        encoding = self.csv_enc.get_active_text() or 'utf_8_sig'
         locale_format = self.csv_locale.get_active()
 
         try:
@@ -388,22 +404,22 @@ class WinExport(WinCSV):
                 open(fname, 'w', encoding=encoding, newline=''),
                 quotechar=self.get_quotechar(),
                 delimiter=self.get_delimiter())
-            if self.add_field_names.get_active():
-                writer.writerow(fields)
             for row, path in zip_longest(data, paths or []):
                 indent = len(path) - 1 if path else 0
                 if row:
                     writer.writerow(self.format_row(
                             row, indent=indent, locale_format=locale_format))
             if popup:
-                if len(data) == 1:
-                    common.message(_('%d record saved.') % len(data))
+                size = len(data)
+                if header:
+                    size -= 1
+                if size <= 1:
+                    common.message(_('%d record saved.') % size)
                 else:
-                    common.message(_('%d records saved.') % len(data))
+                    common.message(_('%d records saved.') % size)
             return True
-        except IOError as exception:
-            common.warning(_("Operation failed.\nError message:\n%s")
-                % exception, _('Error'))
+        except (IOError, UnicodeEncodeError, csv.Error) as exception:
+            common.warning(str(exception), _('Export failed'))
             return False
 
     @classmethod
@@ -414,6 +430,7 @@ class WinExport(WinCSV):
                 if isinstance(val, Number):
                     val = locale.str(val)
                 elif isinstance(val, datetime.datetime):
+                    val = common.timezoned_date(val)
                     val = val.strftime(common.date_format() + ' %X')
                 elif isinstance(val, datetime.date):
                     val = val.strftime(common.date_format())
@@ -426,6 +443,8 @@ class WinExport(WinCSV):
                 val = int(val)
             if i == 0 and indent and isinstance(val, str):
                 val = '  ' * indent + val
+            if isinstance(val, bytes):
+                val = base64.b64encode(val).decode('utf-8')
             row.append(val)
         return row
 
